@@ -18,7 +18,7 @@ import types
 import warnings
 import struct
 
-import pymongo
+import helpers
 import message
 from objectid import ObjectId
 from cursor import Cursor
@@ -56,10 +56,13 @@ class Collection(object):
 
         if not name or ".." in name:
             raise InvalidName("collection names cannot be empty")
-        if "$" in name and not (name in ["$cmd"] or name.startswith("$cmd")):
-            raise InvalidName("collection names must not contain '$'")
+        if "$" in name and not (name.startswith("oplog.$main") or
+                                name.startswith("$cmd")):
+            raise InvalidName("collection names must not "
+                              "contain '$': %r" % name)
         if name[0] == "." or name[-1] == ".":
-            raise InvalidName("collecion names must not start or end with '.'")
+            raise InvalidName("collecion names must not start "
+                              "or end with '.': %r" % name)
 
         self.__database = database
         self.__collection_name = unicode(name)
@@ -183,32 +186,46 @@ class Collection(object):
 
     def update(self, spec, document,
                upsert=False, manipulate=False, safe=False, multi=False):
-        """Update a single document in this collection.
+        """Update a document(s) in this collection.
 
-        Raises TypeError if either spec or document isn't an instance of
-        dict or upsert isn't an instance of bool. If `safe` is True then
-        the update will be checked for errors, raising OperationFailure if one
-        occurred. Safe inserts wait for a response from the database, while
-        normal inserts do not.
+        Raises :class:`TypeError` if either `spec` or `document` is not an
+        instance of ``dict`` or `upsert` is not an instance of ``bool``. If
+        `safe` is ``True`` then the update will be checked for errors, raising
+        :class:`~pymongo.errors.OperationFailure` if one occurred. Safe inserts
+        wait for a response from the database, while normal inserts do not.
+
+        There are many useful `update modifiers`_ which can be used when
+        performing updates. For example, here we use the ``"$set"`` modifier to
+        modify some fields in a matching document::
+
+          >>> db.test.insert({"x": "y", "a": "b"})
+          ObjectId('...')
+          >>> list(db.test.find())
+          [{u'a': u'b', u'x': u'y', u'_id': ObjectId('...')}]
+          >>> db.test.update({"x": "y"}, {"$set": {"a": "c"}})
+          >>> list(db.test.find())
+          [{u'a': u'c', u'x': u'y', u'_id': ObjectId('...')}]
 
         :Parameters:
-          - `spec`: a SON object specifying elements which must be present for
-            a document to be updated
-          - `document`: a SON object specifying the document to be used for the
-            update or (in the case of an upsert) insert. See docs on MongoDB
-            `update modifiers <http://www.mongodb.org/display/DOCS/Updating>`_
+          - `spec`: a ``dict`` or :class:`~pymongo.son.SON` instance specifying
+            elements which must be present for a document to be updated
+          - `document`: a ``dict`` or :class:`~pymongo.son.SON` instance
+            specifying the document to be used for the update or (in the case
+            of an upsert) insert - see docs on MongoDB `update modifiers`_
           - `upsert` (optional): perform an upsert operation
           - `manipulate` (optional): manipulate the document before updating?
           - `safe` (optional): check that the update succeeded?
           - `multi` (optional): update all documents that match `spec`, rather
             than just the first matching document. The default value for
-            `multi` is currently False, but this might eventually change to
-            True. It is recommended that you specify this argument explicitly
-            for all update operations in order to prepare your code for that
-            change.
+            `multi` is currently ``False``, but this might eventually change to
+            ``True``. It is recommended that you specify this argument
+            explicitly for all update operations in order to prepare your code
+            for that change.
 
         .. versionadded:: 1.1.1
            The `multi` parameter.
+
+        .. _update modifiers: http://www.mongodb.org/display/DOCS/Updating
         """
         if not isinstance(spec, types.DictType):
             raise TypeError("spec must be an instance of dict")
@@ -224,24 +241,43 @@ class Collection(object):
             message.update(self.full_name(), upsert, multi,
                            spec, document, safe), safe)
 
-    def remove(self, spec_or_object_id, safe=False):
-        """Remove an object(s) from this collection.
+    def remove(self, spec_or_object_id=None, safe=False):
+        """Remove a document(s) from this collection.
 
-        Raises TypeEror if the argument is not an instance of
-        (dict, ObjectId). If `safe` is True then the remove will be checked for
-        errors, raising OperationFailure if one occurred. Safe removes wait for
-        a response from the database, while normal removes do not.
+        .. warning:: Calls to :meth:`remove` should be performed with
+           care, as removed data cannot be restored.
 
+        Raises :class:`~pymongo.errors.TypeError` if
+        `spec_or_object_id` is not an instance of (``dict``,
+        :class:`~pymongo.objectid.ObjectId`). If `safe` is ``True``
+        then the remove operation will be checked for errors, raising
+        :class:`~pymongo.errors.OperationFailure` if one
+        occurred. Safe removes wait for a response from the database,
+        while normal removes do not.
+
+        If no `spec_or_object_id` is given all documents in this
+        collection will be removed. This is not equivalent to calling
+        :meth:`~pymongo.database.Database.drop_collection`, however, as
+        indexes will not be removed.
 
         :Parameters:
-          - `spec_or_object_id` (optional): a SON object specifying elements
-            which must be present for a document to be removed OR an instance
-            of ObjectId to be used as the value for an _id element
+          - `spec_or_object_id` (optional): a ``dict`` or
+            :class:`~pymongo.son.SON` instance specifying which documents
+            should be removed; or an instance of
+            :class:`~pymongo.objectid.ObjectId` specifying the value of the
+            ``_id`` field for the document to be removed
           - `safe` (optional): check that the remove succeeded?
+
+        .. versionchanged:: 1.2
+           The `spec_or_object_id` parameter is now optional. If it is
+           not specified *all* documents in the collection will be
+           removed.
         """
         spec = spec_or_object_id
+        if spec is None:
+            spec = {}
         if isinstance(spec, ObjectId):
-            spec = SON({"_id": spec})
+            spec = {"_id": spec}
 
         if not isinstance(spec, types.DictType):
             raise TypeError("spec must be an instance of dict, not %s" %
@@ -382,6 +418,9 @@ class Collection(object):
 
     def count(self):
         """Get the number of documents in this collection.
+
+        To get the number of documents matching a specific query use
+        :meth:`pymongo.cursor.Cursor.count`.
         """
         return self.find().count()
 
@@ -394,9 +433,10 @@ class Collection(object):
         """Creates an index on this collection.
 
         Takes either a single key or a list of (key, direction) pairs.
-        The key(s) must be an instance of (str, unicode),
-        and the directions must be one of (`pymongo.ASCENDING`,
-        `pymongo.DESCENDING`). Returns the name of the created index.
+        The key(s) must be an instance of ``(str, unicode)``, and the
+        directions must be one of (:data:`~pymongo.ASCENDING`,
+        :data:`~pymongo.DESCENDING`). Returns the name of the created
+        index.
 
         :Parameters:
           - `key_or_list`: a single key or a list of (key, direction) pairs
@@ -404,8 +444,8 @@ class Collection(object):
           - `direction` (optional): DEPRECATED this option will be removed
           - `unique` (optional): should this index guarantee uniqueness?
           - `ttl` (optional): time window (in seconds) during which this index
-            will be recognized by subsequent calls to `ensure_index` - see
-            documentation for `ensure_index` for details
+            will be recognized by subsequent calls to :meth:`ensure_index` -
+            see documentation for :meth:`ensure_index` for details
         """
         if not isinstance(key_or_list, (str, unicode, list)):
             raise TypeError("key_or_list must either be a single key or a list of (key, direction) pairs")
@@ -417,11 +457,11 @@ class Collection(object):
                           DeprecationWarning)
 
         to_save = SON()
-        keys = pymongo._index_list(key_or_list)
+        keys = helpers._index_list(key_or_list)
         name = self._gen_index_name(keys)
         to_save["name"] = name
         to_save["ns"] = self.full_name()
-        to_save["key"] = pymongo._index_document(keys)
+        to_save["key"] = helpers._index_document(keys)
         to_save["unique"] = unique
 
         self.database().connection()._cache_index(self.__database.name(),
@@ -435,26 +475,28 @@ class Collection(object):
     def ensure_index(self, key_or_list, direction=None, unique=False, ttl=300):
         """Ensures that an index exists on this collection.
 
-        Takes either a single key or a list of (key, direction) pairs.
-        The key(s) must be an instance of (str, unicode),
-        and the direction(s) must be one of (`pymongo.ASCENDING`,
-        `pymongo.DESCENDING`).
+        Takes either a single key or a list of (key, direction)
+        pairs.  The key(s) must be an instance of ``(str, unicode)``,
+        and the direction(s) must be one of
+        (:data:`~pymongo.ASCENDING`, :data:`~pymongo.DESCENDING`).
 
-        Unlike `create_index`, which attempts to create an index
-        unconditionally, `ensure_index` takes advantage of some caching within
-        the driver such that it only attempts to create indexes that might
-        not already exist. When an index is created (or ensured) by PyMongo
-        it is "remembered" for `ttl` seconds. Repeated calls to `ensure_index`
-        within that time limit will be lightweight - they will not attempt to
+        Unlike :meth:`create_index`, which attempts to create an index
+        unconditionally, :meth:`ensure_index` takes advantage of some
+        caching within the driver such that it only attempts to create
+        indexes that might not already exist. When an index is created
+        (or ensured) by PyMongo it is "remembered" for `ttl`
+        seconds. Repeated calls to :meth:`ensure_index` within that
+        time limit will be lightweight - they will not attempt to
         actually create the index.
 
-        Care must be taken when the database is being accessed through multiple
-        connections at once. If an index is created using PyMongo and then
-        deleted using another connection any call to `ensure_index` within the
-        cache window will fail to re-create the missing index.
+        Care must be taken when the database is being accessed through
+        multiple connections at once. If an index is created using
+        PyMongo and then deleted using another connection any call to
+        :meth:`ensure_index` within the cache window will fail to
+        re-create the missing index.
 
-        Returns the name of the created index if an index is actually created.
-        Returns None if the index already exists.
+        Returns the name of the created index if an index is actually
+        created. Returns ``None`` if the index already exists.
 
         :Parameters:
           - `key_or_list`: a single key or a list of (key, direction) pairs
@@ -462,7 +504,7 @@ class Collection(object):
           - `direction` (optional): DEPRECATED this option will be removed
           - `unique` (optional): should this index guarantee uniqueness?
           - `ttl` (optional): time window (in seconds) during which this index
-            will be recognized by subsequent calls to `ensure_index`
+            will be recognized by subsequent calls to :meth:`ensure_index`
         """
         if not isinstance(key_or_list, (str, unicode, list)):
             raise TypeError("key_or_list must either be a single key or a list of (key, direction) pairs")
@@ -473,7 +515,7 @@ class Collection(object):
                           "for a direction on a single key index",
                           DeprecationWarning)
 
-        keys = pymongo._index_list(key_or_list)
+        keys = helpers._index_list(key_or_list)
         name = self._gen_index_name(keys)
         if self.database().connection()._cache_index(self.__database.name(),
                                                      self.name(),
@@ -670,16 +712,56 @@ class Collection(object):
         """Get a list of distinct values for `key` among all documents in this
         collection.
 
+        Raises :class:`TypeError` if `key` is not an instance of
+        ``(str, unicode)``.
+
+        To get the distinct values for a key in the result set of a query
+        use :meth:`pymongo.cursor.Cursor.distinct`.
+
         :Parameters:
           - `key`: name of key for which we want to get the distinct values
 
+        .. note:: Requires server version **>= 1.1.0**
+
         .. versionadded:: 1.1.1
         """
-        if not isinstance(key, types.StringTypes):
-            raise TypeError("key must be an instance of (str, unicode)")
+        return self.find().distinct(key)
 
-        command = SON([("distinct", self.__collection_name), ("key", key)])
-        return self.__database._command(command)["values"]
+    def map_reduce(self, map, reduce, full_response=False, **kwargs):
+        """Perform a map/reduce operation on this collection.
+
+        If `full_response` is ``False`` (default) returns a
+        :class:`~pymongo.collection.Collection` instance containing
+        the results of the operation. Otherwise, returns the full
+        response from the server to the `map reduce command`_.
+
+        :Parameters:
+          - `map`: map function (as a JavaScript string)
+          - `reduce`: reduce function (as a JavaScript string)
+          - `full_response` (optional): if ``True``, return full response to
+            this command - otherwise just return the result collection
+          - `**kwargs` (optional): additional arguments to the
+            `map reduce command`_ may be passed as keyword arguments to this
+            helper method, e.g.::
+
+            >>> db.test.map_reduce(map, reduce, limit=2)
+
+        .. note:: Requires server version **>= 1.1.1**
+
+        .. seealso:: :doc:`/examples/map_reduce`
+
+        .. versionadded:: 1.2
+
+        .. _map reduce command: http://www.mongodb.org/display/DOCS/MapReduce
+        """
+        command = SON([("mapreduce", self.__collection_name),
+                       ("map", map), ("reduce", reduce)])
+        command.update(**kwargs)
+
+        response = self.__database._command(command)
+        if full_response:
+            return response
+        return self.__database[response["result"]]
 
     def __iter__(self):
         return self
