@@ -18,7 +18,7 @@ import types
 import struct
 import warnings
 
-import pymongo
+import helpers
 import message
 from son import SON
 from code import Code
@@ -121,13 +121,7 @@ class Cursor(object):
 
     def __query_spec(self):
         """Get the spec to use for a query.
-
-        Just `self.__spec`, unless this cursor needs special query fields, like
-        orderby.
         """
-        if not self.__ordering and not self.__explain and not self.__hint:
-            return self.__spec
-
         spec = SON({"query": self.__spec})
         if self.__ordering:
             spec["orderby"] = self.__ordering
@@ -194,24 +188,28 @@ class Cursor(object):
     def __getitem__(self, index):
         """Get a single document or a slice of documents from this cursor.
 
-        Raises InvalidOperation if this cursor has already been used.
+        Raises :class:`~pymongo.errors.InvalidOperation` if this
+        cursor has already been used.
 
-        To get a single document use an integral index, e.g.:
+        To get a single document use an integral index, e.g.::
 
-        >>> db.test.find()[50]
+          >>> db.test.find()[50]
 
-        An IndexError will be raised if the index is negative or greater than
-        the amount of documents in this cursor.
+        An :class:`IndexError` will be raised if the index is negative
+        or greater than the amount of documents in this cursor. Any
+        limit applied to this cursor will be ignored.
 
-        To get a slice of documents use a slice index, e.g.:
+        To get a slice of documents use a slice index, e.g.::
 
-        >>> db.test.find()[20:25]
+          >>> db.test.find()[20:25]
 
-        This will return this cursor with a limit of 5 and skip of 20 applied.
-        Using a slice index will override any prior limits or skips applied to
-        this cursor (including those applied through previous calls to this
-        method). Raises IndexError when the slice has a step, a negative start
-        value, or a stop value less than or equal to the start value.
+        This will return this cursor with a limit of ``5`` and skip of
+        ``20`` applied.  Using a slice index will override any prior
+        limits or skips applied to this cursor (including those
+        applied through previous calls to this method). Raises
+        :class:`IndexError` when the slice has a step, a negative
+        start value, or a stop value less than or equal to the start
+        value.
 
         :Parameters:
           - `index`: An integer or slice index to be applied to this cursor
@@ -242,7 +240,7 @@ class Cursor(object):
             if index < 0:
                 raise IndexError("Cursor instances do not support negative indices")
             clone = self.clone()
-            clone.skip(index)
+            clone.skip(index + self.__skip)
             clone.limit(-1) # use a hard limit
             for doc in clone:
                 return doc
@@ -253,21 +251,23 @@ class Cursor(object):
         """Sorts this cursor's results.
 
         Takes either a single key and a direction, or a list of (key,
-        direction) pairs. The key(s) must be an instance of (str, unicode), and
-        the direction(s) must be one of (`pymongo.ASCENDING`,
-        `pymongo.DESCENDING`). Raises InvalidOperation if this cursor has
-        already been used. Only the last `sort` applied to this cursor has any
-        effect.
+        direction) pairs. The key(s) must be an instance of ``(str,
+        unicode)``, and the direction(s) must be one of
+        (:data:`~pymongo.ASCENDING`,
+        :data:`~pymongo.DESCENDING`). Raises
+        :class:`~pymongo.errors.InvalidOperation` if this cursor has
+        already been used. Only the last :meth:`sort` applied to this
+        cursor has any effect.
 
         :Parameters:
-          - `key_or_list`: a single key or a list of (key, direction) pairs
-            specifying the keys to sort on
-          - `direction` (optional): only used if key_or_list is a single
-            key, if not given ASCENDING is assumed
+          - `key_or_list`: a single key or a list of (key, direction)
+            pairs specifying the keys to sort on
+          - `direction` (optional): only used if `key_or_list` is a single
+            key, if not given :data:`~pymongo.ASCENDING` is assumed
         """
         self.__check_okay_to_chain()
-        keys = pymongo._index_list(key_or_list, direction)
-        self.__ordering = pymongo._index_document(keys)
+        keys = helpers._index_list(key_or_list, direction)
+        self.__ordering = helpers._index_document(keys)
         return self
 
     def count(self, with_limit_and_skip=False):
@@ -307,6 +307,32 @@ class Cursor(object):
             return 0
         return int(response["n"])
 
+    def distinct(self, key):
+        """Get a list of distinct values for `key` among all documents in the
+        result set of this query.
+
+        Raises :class:`TypeError` if `key` is not an instance of
+        ``(str, unicode)``.
+
+        :Parameters:
+          - `key`: name of key for which we want to get the distinct values
+
+        .. note:: Requires server version **>= 1.1.3+**
+
+        .. seealso:: :meth:`pymongo.collection.Collection.distinct`
+
+        .. versionadded:: 1.2
+        """
+        if not isinstance(key, types.StringTypes):
+            raise TypeError("key must be an instance of (str, unicode)")
+
+        command = SON([("distinct", self.__collection.name()), ("key", key)])
+
+        if self.__spec:
+            command["query"] = self.__spec
+
+        return self.__collection.database()._command(command)["values"]
+
     # __len__ is deprecated (replaced with count(True)) and will be removed.
     #
     # The reason for this deprecation is a bit complex:
@@ -336,16 +362,20 @@ class Cursor(object):
     def hint(self, index):
         """Adds a 'hint', telling Mongo the proper index to use for the query.
 
-        Judicious use of hints can greatly improve query performance. When
-        doing a query on multiple fields (at least one of which is indexed)
-        pass the indexed field as a hint to the query. Hinting will not do
-        anything if the corresponding index does not exist. Raises
-        InvalidOperation if this cursor has already been used.
+        Judicious use of hints can greatly improve query
+        performance. When doing a query on multiple fields (at least
+        one of which is indexed) pass the indexed field as a hint to
+        the query. Hinting will not do anything if the corresponding
+        index does not exist. Raises
+        :class:`~pymongo.errors.InvalidOperation` if this cursor has
+        already been used.
 
-        `index` should be an index as passed to create_index
-        (e.g. [('field', ASCENDING)]). If `index` is None any existing
-        hints for this query are cleared. The last hint applied to this cursor
-        takes precedence over all others.
+        `index` should be an index as passed to
+        :meth:`~pymongo.collection.Collection.create_index`
+        (e.g. ``[('field', ASCENDING)]``). If `index`
+        is ``None`` any existing hints for this query are cleared. The
+        last hint applied to this cursor takes precedence over all
+        others.
 
         :Parameters:
           - `index`: index to hint on (as an index specifier)
@@ -357,7 +387,7 @@ class Cursor(object):
 
         if not isinstance(index, (types.ListType)):
             raise TypeError("hint takes a list specifying an index")
-        self.__hint = pymongo._index_document(index)
+        self.__hint = helpers._index_document(index)
         return self
 
     def where(self, code):
@@ -402,7 +432,11 @@ class Cursor(object):
 
         self.__connection_id = connection_id
 
-        response = pymongo.Connection._unpack_response(response, self.__id)
+        try:
+            response = helpers._unpack_response(response, self.__id)
+        except AutoReconnect:
+            db.connection()._reset()
+            raise
         self.__id = response["cursor_id"]
 
         # starting from doesn't get set on getmore's for tailable cursors
