@@ -29,7 +29,7 @@ from pymongo.objectid import ObjectId
 from pymongo.code import Code
 from pymongo.binary import Binary
 from pymongo.collection import Collection
-from pymongo.errors import InvalidName, OperationFailure
+from pymongo.errors import InvalidName, OperationFailure, InvalidDocument
 from pymongo import ASCENDING, DESCENDING
 from pymongo.son import SON
 
@@ -300,6 +300,17 @@ class TestCollection(unittest.TestCase):
         qcheck.check_unittest(self, remove_insert_find_one,
                               qcheck.gen_mongo_dict(3))
 
+    def test_remove_all(self):
+        self.db.test.remove()
+        self.assertEqual(0, self.db.test.count())
+
+        self.db.test.insert({"x": 1})
+        self.db.test.insert({"y": 1})
+        self.assertEqual(2, self.db.test.count())
+
+        self.db.test.remove()
+        self.assertEqual(0, self.db.test.count())
+
     def test_find_w_fields(self):
         db = self.db
         db.test.remove({})
@@ -820,6 +831,59 @@ class TestCollection(unittest.TestCase):
 
         self.assertEqual(["a", "b", "c"], distinct)
 
+    def test_query_on_query_field(self):
+        self.db.drop_collection("test")
+        self.db.test.save({"query": "foo"})
+        self.db.test.save({"bar": "foo"})
+
+        self.assertEqual(1, self.db.test.find({"query": {"$ne": None}}).count())
+        self.assertEqual(1, len(list(self.db.test.find({"query": {"$ne": None}}))))
+
+    def test_insert_large_document(self):
+        self.assertRaises(InvalidDocument, self.db.test.insert,
+                          {"foo": "x" * 4 * 1024 * 1024})
+        self.assertRaises(InvalidDocument, self.db.test.insert,
+                          [{"x": 1}, {"foo": "x" * 4 * 1024 * 1024}])
+        self.db.test.insert([{"foo": "x" * 2 * 1024 * 1024},
+                             {"foo": "x" * 2 * 1024 * 1024}], safe=True)
+
+    def test_map_reduce(self):
+        if not version.at_least(self.db.connection(), (1, 1, 1)):
+            raise SkipTest()
+
+        db = self.db
+        db.drop_collection("test")
+
+        db.test.insert({"id": 1, "tags": ["dog", "cat"]})
+        db.test.insert({"id": 2, "tags": ["cat"]})
+        db.test.insert({"id": 3, "tags": ["mouse", "cat", "dog"]})
+        db.test.insert({"id": 4, "tags": []})
+
+        map = Code("function () {"
+                   "  this.tags.forEach(function(z) {"
+                   "    emit(z, 1);"
+                   "  });"
+                   "}")
+        reduce = Code("function (key, values) {"
+                      "  var total = 0;"
+                      "  for (var i = 0; i < values.length; i++) {"
+                      "    total += values[i];"
+                      "  }"
+                      "  return total;"
+                      "}")
+        result = db.test.map_reduce(map, reduce)
+        self.assertEqual(3, result.find_one({"_id": "cat"})["value"])
+        self.assertEqual(2, result.find_one({"_id": "dog"})["value"])
+        self.assertEqual(1, result.find_one({"_id": "mouse"})["value"])
+
+        full_result = db.test.map_reduce(map, reduce, full_response=True)
+        self.assertEqual(6, full_result["counts"]["emit"])
+
+        result = db.test.map_reduce(map, reduce, limit=2)
+        self.assertEqual(2, result.find_one({"_id": "cat"})["value"])
+        self.assertEqual(1, result.find_one({"_id": "dog"})["value"])
+        self.assertEqual(None, result.find_one({"_id": "mouse"}))
+
 
 if __name__ == "__main__":
     unittest.main()
